/*
***********************************************************
$SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
$HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/analysis.c $
$LastChangedRevision: 45 $
$Date: 2014-06-12 19:31:23 -0400 (Thu, 12 Jun 2014) $
$Author: olangthaler $
***********************************************************

 Previously part of SuperSNARK
 */

/*-------------------------- analysis.c ----------------------------------
        Super Snark V 1.0
        written by Jingsheng Zheng
        Modified by Jolyon Browne
  ------------------------------------------------------------------------*/
#include <strings.h>

#include "experimenter.h"
#include "errorc.h"
#include "analysis.h"

/*-------------------------- analysis.c ----------------------------------
        This function sets up the appropriate command lines needed for
  SNARK's EVALUATE command.

INPUTS:
  ana_flname - name of file containing comparisons to be made.
  structure - array containing structures to be evaluated.
  nstr - number of structures in array 'structure.'
  iter_flag - iteration_flag_line needed by EVALUATE command

OUTPUTS:
  none
  ------------------------------------------------------------------------*/

void evaluate(char* ana_flname, char *** structure, int nstr, char* iter_flag)
{
    FILE *anafl;
    float lower, upper;
    int i, j = 3; // added "j". Lajos, Dec 13, 2004
    char string[MAXLINESIZE];


    if ((anafl = fopen(ana_flname, "r")) == NULL) {
        errorc("in analysis.c: error in opening file ", ana_flname);
    }

    fprintf(pstream, "EVALUATE RESOLUTION\n");
    fprintf(pstream, "experimenter evaluation 1\n"); //added by Lajos, Dec 13, 2004
    fprintf(pstream, "SELECTIVE\n");
    for (i = 0; i < nstr; i++) {
        fprintf(pstream, "%s\n", (*structure)[i]);
    }
    fprintf(pstream, "LAST\n");
    fprintf(pstream, "%s\n", iter_flag);

    fgets(string, sizeof (string), anafl); /* don't want first line */
    while ( isSkip(string) )     //jump over comment lines and blank lines
    {
        if ( fgets(string, sizeof (string), anafl) == NULL) break;
    }
    fgets(string, sizeof (string), anafl); /* second line is either a
					MODE command or a COMPARE command */
    while ( isSkip(string) )     //jump over comment lines and blank lines
    {
        if ( fgets(string, sizeof (string), anafl) == NULL) break;
    }
    if (strncasecmp(string, "mode", 4) == 0) {
        sscanf(string, "%*s%f%f", &lower, &upper);
        fprintf(pstream, "MODE LOWER %f UPPER %f\n", lower, upper);
    }

    fprintf(pstream, "EVALUATE RESOLUTION\n");
    fprintf(pstream, "experimenter evaluation 2\n"); //added by Lajos, Dec 13, 2004
    fprintf(pstream, "WHOLEPIC\n");
    fprintf(pstream, "%s\n", iter_flag);

    /* read in MODE values for clipping with WHOLEPIC command */
    while ((fgets(string, sizeof (string), anafl)) != NULL) {
        if (strncasecmp(string, "mode", 4) == 0) {
            sscanf(string, "%*s%f%f", &lower, &upper);
            fprintf(pstream, "MODE LOWER %f UPPER %f\n", lower, upper);
            fprintf(pstream, "EVALUATE RESOLUTION\n");
            fprintf(pstream, "experimenter evaluation %d\n", j++); //added by Lajos, Dec 13, 2004
            fprintf(pstream, "WHOLEPIC\n");
            fprintf(pstream, "%s\n", iter_flag);
        }
    }
    fclose(anafl);
}
