/*
**********************************************************************
$SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
$HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/creapr.cpp $
$LastChangedRevision: 85 $
$Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
$Author: agulati $
**********************************************************************

 * THIS SUBROUTINE GENERATES PROJECTION DATA
 * Modified to simulate emission tomography if quanin = 4 by D. Odhner
 */

#include <cstdlib>
#include <cstdio>
#include <cmath>
#include <time.h>

#include "blkdta.h"
#include "creacm.h"
#include "geom.h"
#include "spctrm.h"
#include "noise.h"
#include "consts.h"
#include "uiod.h"

#include <DIGGauss.h>
#include <DIGPoisson.h>
#include "posit.h"
#include "anglst.h"
#include "raylen.h"
#include "file11.h"
#include "anglst.h"

#include "creapr.h"

//jk 11/18/2007 introducing variability
#include "ray.h"
#include "wray.h"


// bug 221 - moved noise stuff to noise.cpp - swr - 3/2/07

void creapr(  ) {

    REAL sum;
    REAL variability_subray_sum;

    REAL theta, sinth, costh, raysum;
    REAL weight;
    REAL strpwt;

    INTEGER i, k, n, nnr, np, l, nr, nb;

    //jk 11/18/2007 introducing variability
    //variables needed for computation of projections through 
    INTEGER * list;
    list = new INTEGER[2 * GeoPar.nelem];
    REAL * weight_pix;
    weight_pix = new REAL[2 * GeoPar.nelem];
    INTEGER * numb;
    numb = new INTEGER;
    REAL * snorm;
    snorm = new REAL;


    REAL* ind;
    REAL* nlo;

    REAL emr[13][7], ax, ay, mx, my;
    REAL* pbase = NULL; //wei 3/2005

    // COMPUTE PROJECTION BY PROJECTION  HENCE WE NEED SPACE FOR
    // ONLY ONE PROJECTION OF LENGTH NRAYS
    // NLO WILL DENOTE THE FIRST RAY AND NHI THE LAST RAY


    pbase = new REAL[GeoPar.usrays];
    nlo = pbase;

    //jk 09/28/2008 start the seed for noise generation here, so it is not
    //influenced by variability number generation
    
    //if any type of noise is used
    if (NoisePar.quanin != NONOISE || NoisePar.addnfl 
            || NoisePar.ultnfl || NoisePar.sctnfl) {
        //then set the seed
        if (Creacm.iseed < 0) Srand(time(NULL));     // bug 184 - swr - 11/04/05
        else if (Creacm.iseed > 0) Srand(Creacm.iseed);
        else Srand(1);  //set the default seed so that the default sequence 
                        //is the same regardless of presence of variability before
    }

    
    //jk 11/18/2007 introducing variability
    //stores the value of projection, written to the FILE11 after each projection
    //hence we need only one
    REAL * pseudoray = NULL;
    pseudoray = new REAL[GeoPar.usrays];


    // SUBDIVIDE RAYS FOR AVERAGING OVER  NAVE2 SUBRAYS/SUBSTRIPS
    // CHANGE PINC, NRAYS AND MIDRAY IN /GEOM/ FOR USE BY POSIT.
    // WHEN RDPROJ IS DONE THEESE VALUES WILL BE RESTORED
    // SAVE PINC IN WEIGHT FOR STRIP CASE

    weight = GeoPar.pinc;
    strpwt = weight;
    GeoPar.pinc /= GeoPar.nave2;
    GeoPar.nrays = GeoPar.usrays * GeoPar.nave2;
    GeoPar.midray = GeoPar.nrays / 2;

    // STORE SINE AND COSINE OF ANGLE SUBTENDED BY EACH SUBRAY AT SOURCE
    // FOR DIVERGENT CASE ONLY FOR USE BY 'POSIT'

    if (GeoPar.div) Anglst.genphi();


    // SELECT PROJECTION BY PROJECTION AND CALCULATE PROJECTION DATA

    // IF PHANTOM IS BLANK ATTENUATION IS THAT DUE TO BACKGROUND

    for (n = 0; n < GeoPar.nave2; n++) 
	{
        for (l = 0; l < Spctrm.nergy; l++) 
		{
            emr[n][l] = NoisePar.raysum(Spctrm.backgr[l]);
        }
    }

    for (np = 0; np < GeoPar.prjnum; np++) { //230
        // IF CALIBRATION TYPE 1   GET THE CALIBRATION MEASUREMENT HERE

        Anglst.getang(np, &theta, &sinth, &costh);

        if (GeoPar.vri) strpwt = (REAL) (MAX0(fabs(sinth), fabs(costh)) * weight);

        nnr = 0;

        for (nr = 0; nr < GeoPar.usrays; nr++) 
		{

            if (!Creacm.modif) 
			{
                if (Creacm.nobj > 0) 
				{

                    // CALCULATE CONTRIBUTION TO ATTENUATION BY EACH OBJECT FOR
                    // EACH SUBRAY (1 TO NRAYS)

                    for (n = 0; n < GeoPar.nave2; n++) 
					{

                        posit(np, nnr, &ax, &ay, &mx, &my);
                        nnr++;

                        for (k = 0; k < Creacm.nobj; k++) 
						{
#ifdef FFCOMPARE
							
                            REAL raylen_temp = raylen(objects[k].type, ax, ay, mx, my, objects[k].cx, objects[k].cy, objects[k].u, objects[k].v, objects[k].cosang, objects[k].sinang);

                            // rounding !!!
                            raylen_temp = (REAL) ((INTEGER) ((REAL) 10.0 * raylen_temp)) / (REAL) 10.0;

                            objects[k].rayl = raylen_temp;

#else
                            objects[k].rayl = raylen(objects[k].type, ax, ay, mx, my, objects[k].cx, objects[k].cy, objects[k].u, objects[k].v, objects[k].cosang, objects[k].sinang);
#endif
                        }
                        
                        
                        //////////////////////////////////////////////////////////////////////////////////////////
                        //if variability is set then we need to add line integrals through variability data
                        //to calculations of projection data
                        if (variab_set) 
						{
                            
							//                            get information about the pixels through which the ray passes
							//                            (it will be used below when the contributions are computed 
							//                            for each energy level):
							//                            
							//                            numb      THE NUMBER OF PIXELS A RAY PASSES THROUGH
							//                            snorm     SUM OF SQUARES OF WEIGHTS
							//                            weight_pix    THE LENGTH OF A RAY IN A PIXEL(CORRESPONDING TO
							//                                      ENTRY IN THE ARRAY LIST)
							//                            list      THE LIST OF INDECIES INTO THE ARRAY variab_array THAT CORRESP
							//                                      TO THE SILECTED PIXELS
							//                            values of ax, ay, mx and my are reused from a previous call to 
							//                            posit(np, nnr, &ax, &ay, &mx, &my);
                            wray(np, nnr, list, weight_pix, numb, snorm, &ax, &ay, &mx, &my);
                            
                        }

                        //////////////////////////////////////////////////////////////////////////////////////////                    
                        //add together all the contributions to the subray    
                        for (l = 0; l < Spctrm.nergy; l++) 
						{
                            //background absorption
                            raysum = Spctrm.backgr[l];
                            //variability (if present)
                            if (variab_set) 
							{
                                variability_subray_sum = 0.0;
                                //if the ray went through any pixels compute the contribution to the integral
                                if (*numb != 0) 
								{
                                    //for each pixels through which the ray goes
                                    for (nb = 0; nb < *numb; nb++) 
									{
                                        //add the contribution of the pixel to subray at energy level l
                                         variability_subray_sum += variab_array[l][list[nb]] * weight_pix[nb];
                                    }
                                }
                                
                                raysum += variability_subray_sum; //jk 5/21/2008  add the line ray sum for variability
                            }
                            //contributions from all the objects
                            for (k = 0; k < Creacm.nobj; k++) 
							{
                                raysum += objects[k].den1[l] * objects[k].rayl;
                            }

                            emr[n][l] = NoisePar.raysum(raysum);
                        } // for l
                    }  //for n
                } //if (Creacm.nobj > 0) 
            } //if (!Creacm.modif)

            
            // CALCULATE NO. OF PHOTONS OR ABSORPTION WEIGHTED BY APERTURE WT
            // AND THE SPECTRAL DISTRIBUTION

            sum = 0.0;
            for (n = 0; n < GeoPar.nave2; n++) 
			{
                for (l = 0; l < Spctrm.nergy; l++) 
				{
                    sum += Creacm.aperwt[n] * Spctrm.engwt[l] * emr[n][l];
                }
            }


            // APPLY QUANTUM NOISE
            pbase[nr] = NoisePar.applyQuantumNoise(np, nr, sum);

        }

        // COMPUTE ATTENUATION
        NoisePar.scatterNoise(pbase);


        // GET ACTUAL ATTENUATION AFTER CALIBRATION AND APPLY ADDITIVE, MULT
        // NOISE
        ind = pbase;
        for (nr = 0; nr < GeoPar.usrays; nr++) 
		{
            *ind = NoisePar.computeAttenuation(np, nr, *ind);
            if (GeoPar.strip) *ind *= strpwt;

            *ind = NoisePar.multiplicativeNoise(*ind);
            *ind = NoisePar.additiveNoise(*ind);


            ind++;
        }

        File11.WriteProj(theta, pbase, GeoPar.usrays);

    }

    File11.WriteEndOfProj();

    //Clean up all the allocated memory
    if (pbase != NULL) delete[] pbase;
    if (list != NULL) delete [] list;
    if (weight_pix != NULL) delete [] weight_pix;
    if (numb != NULL) delete numb;
    if (snorm != NULL) delete snorm;
    if (pseudoray != NULL) delete pseudoray;
    
    for (i= 6; i >= 0; i--) 
	{
        if (variab_array[i] != NULL)  delete [] variab_array[i];  //it was allocated in creaph.cpp 
    }
    
    return;
}
