/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/emap.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 ===========================================================================
 maximum a-posteriori probability expectation maximization algorithm
 for the reconstruction in emission tomography (Herman, De Pierro, and
 Gai, "On Methods for Maximum A Posteriori Image Reconstruction with
 a Normal Prior," Technical Report MIPG181, Medical Image Processing
 Group, Dept. of Radiology, Univ. of Pennsylvania, 1992.)

 input line in snark:
 gamma  [eval sy1 sy2]]
 e.g.:  1.0    eval 10  1
 
 ===========================================================================
 */

#include <cstdio>
#include <cstdlib>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "blob.h"
#include "uiod.h"

#include "emap.h"

INTEGER emap_class::Init()
{
	MAPUser1 = fopen("MAPUser1", "r+");
	MAPEM.qj = NULL;
	MAPEM.Kx = NULL;
	MAPEM.Sx = NULL;
	MAPEM.Wjj = NULL;
	MAPEM.Sjjinv = NULL;
	MAPEM.prj = NULL;                       //wei 3/2005

	return 0;
}

INTEGER emap_class::Reset()
{
	fclose(MAPUser1);
	if (MAPEM.qj != NULL)
		delete[] MAPEM.qj;
	if (MAPEM.Kx != NULL)
		delete[] MAPEM.Kx;
	if (MAPEM.Sx != NULL)
		delete[] MAPEM.Sx;
	if (MAPEM.Wjj != NULL)
		delete[] MAPEM.Wjj;
	if (MAPEM.Sjjinv != NULL)
		delete[] MAPEM.Sjjinv;
	if (MAPEM.prj != NULL)
		delete[] MAPEM.prj;                //wei 3/2005
	return 0;
}

BOOLEAN emap_class::Run(REAL* recon, INTEGER* list, REAL* weight, INTEGER iter)
{
	INTEGER area;
	BOOLEAN flag;  //added hstau 1/04

	if (Blob.pix_basis)
	{
		area = GeoPar.area;
	}
	else
	{
		area = Blob.area;
	}

//------------------------   statements    ----------------------------
	if (iter == 1)
	{

		MAPUser1 = fopen("MAPUser1", "w");

		if (GeoPar.par)
		{
			MAPEM.delta = MAX0((REAL) 1.0,
					(REAL) 2.0 * (GeoPar.pixsiz / GeoPar.pinc));
		}
		else
		{
			MAPEM.delta = GeoPar.pixsiz / (GeoPar.pinc * (REAL) sqrt(2.) * (GeoPar.radius - (REAL) sqrt(.5) * GeoPar.pixsiz * GeoPar.nelem) / GeoPar.stod);
		}

		MAPEM.delta++;

		if (((INTEGER) MAPEM.delta) > GeoPar.usrays)
		{
			MAPEM.delta = (REAL) GeoPar.usrays;
		}

		// nelem not used. hstau
		ReadInp(area, GeoPar.nelem, GeoPar.usrays, GeoPar.prjnum, &flag); //hstau 1/04

		if (flag)
			return TRUE; //added hstau 1/04

		if (MAPEM.gamma1 <= Consts.zero)
		{
			MAPEM.gamma1 = 0.0;
		}

		if (GeoPar.nelem == 1)
			MAPEM.gamma1 = 0.0;                                  //wei 1/2005

		if (!Blob.pix_basis && MAPEM.gamma1 > 0.0)
		{
			fprintf(output,
					"\n *** error: gamma must be 0.0 when using blob basis");
			return TRUE;  //it was exit(1); bug 123, hstau 7/8/05
		}

		InitMAP(recon);
	}  // iter ==1

	for (MAPEM.idy = 0; MAPEM.idy <= MAPEM.idymax - 1; MAPEM.idy++)
	{
		for (MAPEM.idx = 0; MAPEM.idx <= MAPEM.idxmax - 1; MAPEM.idx++)
		{
			if (MAP(recon))
				return TRUE;            //added by wei, 1/2005
		}
	}

	if (MAPEM.mapevl)
	{
		EvalMAP(recon, iter);
	}

	return FALSE;
}
