/* 
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/emap_CEvalMAP.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************


 ===========================================================================
 Evaluation of the MAP algorithms (the results are written to "user1")
 ===========================================================================
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "uiod.h"
#include "wray.h"
#include "blob.h"

#include "emap.h"

void emap_class::EvalMAP(REAL* recon, INTEGER iter)
{
	//------ local variables
	INTEGER i, j, iadr, nb, np, nr, numb, list[4096], usnr;
	REAL snorm, weight[4096], diff1;
	double sum1, sum2, y, diffnbs, totalsum, sclf, smx, pictk, ssum1, ssum2;

	//------ evaluation

	//------ log-likelihood
	sum1 = 0.0;
	sclf = MAPEM.sclfactor;
	for (i = 0; i < GeoPar.prjnum; i++)
	{
		ssum1 = 0.0;
		for (j = 0; j < GeoPar.usrays; j++)
		{
			np = i;
			usnr = j;
			nr = usnr + GeoPar.fusray;

			y = MAPEM.prj[(np) * GeoPar.usrays + usnr] * sclf;

			smx = 0.0;
			if (Blob.pix_basis)
			{
				wray(np, nr, list, weight, &numb, &snorm);
			}
			else
			{
				Blob.bwray(np, nr, list, weight, &numb, &snorm);
			}

			if (numb != 0)
			{
				for (nb = 0; nb < numb; nb++)
				{
					iadr = list[nb];
					pictk = recon[iadr];
					smx += pictk * weight[nb];
				}
			}
			smx *= sclf;

			if (smx > 0.0)
			{
				ssum1 += y * log(smx) - smx;
			}
			else if (smx < 0.0)
			{
				fprintf(output, "\n In EvalMAP  smx = %14.7f y = %14.7f", smx,
						y);
				fprintf(output, "\n smx must be greater than 0.0");
			}

		}
		sum1 += ssum1;
	}

	sum2 = 0.0; // moved out of the `if' block for initialization. Lajos, Feb 10, 2005
	if (Blob.pix_basis)
	{ // when blob, gamma is 0.0, so this step is skept. hstau 6/25/2003

		for (i = GeoPar.nelem; i < GeoPar.area - 2 * GeoPar.nelem + 1; i +=
				GeoPar.nelem)
		{
			ssum2 = 0.0;
			for (j = i + 1; j < i + GeoPar.nelem - 2; j++)
			{
				diff1 = recon[j]
						- (REAL) 0.125
								* (recon[j - 1 - GeoPar.nelem]
										+ recon[j - GeoPar.nelem]
										+ recon[j + 1 - GeoPar.nelem]
										+ recon[j - 1] + recon[j + 1]
										+ recon[j - 1 + GeoPar.nelem]
										+ recon[j + GeoPar.nelem]
										+ recon[j + 1 + GeoPar.nelem]);
				diffnbs = diff1 * MAPEM.sclfactor;
				ssum2 += diffnbs * diffnbs;
			}
			sum2 += ssum2;
		}

		sum2 = -0.5 * MAPEM.gamma1 * sum2;
	}  // if pixel basis

	totalsum = sum1 + sum2;

	//------ write results
	if (iter == 1)
	{
		if (MAPEM.gamma1 == 0.0)
		{
			fprintf(MAPUser1, "\n    iter     log-likelihood");
		}
		else
		{
			///if(MAPEM.gamma1 != 0.0) {
			fprintf(MAPUser1,
					"\n    iter     log-likelihood          log-posterior");
		}
	}

	if (MAPEM.gamma1 == 0.0)
	{
		fprintf(MAPUser1, "\n %5i  %20.9f", iter, sum1);
	}
	else
	{
		///if(MAPEM.gamma1 != 0.0) {
		fprintf(MAPUser1, "\n %5i  %20.9f  %20.9f", iter, sum1, totalsum);
	}

	fflush(MAPUser1);

	return;
}
