/* 
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/emap_CMAP.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 ======================================================================
 one iteration of the maximum a-posteriori reconstruction algorithm
 for emission tomography
 ======================================================================
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "blob.h"

#include "emap.h"

BOOLEAN emap_class::MAP(REAL* recon)
{
	//------ local variables
	INTEGER i, j, first, np, nr, idelta, usnr;
	REAL p, q;
	REAL yi;
	INTEGER area;


	if (Blob.pix_basis)
	{
		area = GeoPar.area;
	}

	else
	{
		area = Blob.area;
	}

	idelta = (INTEGER) (MAPEM.delta);

	//====== step 1: compute q for iteration k+1:
	for (j = 0; j < area; j++)
	{
		MAPEM.qj[j] = 0.0;
	}

	for (np = 0; np < GeoPar.prjnum; np++)
	{
		for (first = 0; first < idelta; first++)
		{
			for (usnr = first; usnr < GeoPar.usrays; usnr += idelta)
			{
				nr = usnr + GeoPar.fusray;
				yi = MAPEM.prj[(np) * GeoPar.usrays + usnr];
				if (yi != 0.0)
				{
					if (findqpart(np, nr, yi, recon, MAPEM.qj))
						return TRUE; //if findqpart return false, EMAP is not applicable, bywei 1/2005
				}
			}
		}
	}

	// MAPEM.idy = MAPEM.idx = 0, see comments in emap.cpp

#ifdef HSTAU
	fprintf(output,"\n #IDY=%d #IDX=%d\n", MAPEM.idy, MAPEM.idx);fflush(output);
#endif

	if (Blob.pix_basis)
	{
		for (i = MAPEM.idy * GeoPar.nelem;
				i < GeoPar.nelem * (GeoPar.nelem - 1) + 1;
				i += GeoPar.nelem * MAPEM.idymax)
		{
			for (j = i + MAPEM.idx; j < i + GeoPar.nelem; j += MAPEM.idxmax)
			{

				if (MAPEM.gamma1 > 0.0)
				{
					MAPEM.qj[j] *= recon[j] * MAPEM.Sjjinv[j] / MAPEM.gamma1;
				}
				else
				{
					if (MAPEM.Wjj[j] > Consts.zero)
						MAPEM.qj[j] *= recon[j] / MAPEM.Wjj[j];
					else
						MAPEM.qj[j] = recon[j]; //keep recon[j] unchanged when qj[j]is zero, by wei 1/2005
				}
			}
		}
	}
	else
	{ //blob case
		for (i = MAPEM.idy * Blob.M; i < Blob.M * (Blob.H - 1) + 1;
				i += Blob.M * MAPEM.idymax)
		{
			for (j = i + MAPEM.idx; j < i + Blob.M; j += MAPEM.idxmax)
			{

				if (j % 2 == Blob.pr)
				{
					if (MAPEM.Wjj[j] > Consts.zero)
						MAPEM.qj[j] *= recon[j] / MAPEM.Wjj[j];
					else
						MAPEM.qj[j] = recon[j]; //recon[j] keep unchanged when Wjj[j] is zero, by wei 1/2005
				}
			}
		}
	}

	if (MAPEM.gamma1 > 0.0)
	{  // blob case will not enter here.hstau
		//====== step 2: compute "smoothing component" from "recon" to "Sx"
		applyK(recon, MAPEM.Kx, GeoPar.nelem);

//     ---Set edges to 0.0
		for (i = 0; i < GeoPar.nelem; i++)
		{
			MAPEM.Kx[i] = 0.0;          // first row
			//MAPEM.Kx[(i-1)*GeoPar.nelem+1] = 0.0;
			MAPEM.Kx[(i + 1) * GeoPar.nelem - 1] = 0.0; // last column
			MAPEM.Kx[i * GeoPar.nelem] = 0.0;     // first column
			MAPEM.Kx[(GeoPar.nelem - 1) * GeoPar.nelem + i] = 0.0; // last row
		}

		applyK(MAPEM.Kx, MAPEM.Sx, GeoPar.nelem);

		//====== step 3: compute x for iteration k+1 from "Sx" and "qj"
		//cvd$     nosync
		for (i = MAPEM.idy * GeoPar.nelem;
				i < GeoPar.nelem * (GeoPar.nelem - 1) + 1;
				i += GeoPar.nelem * MAPEM.idymax)
		{
			for (j = i + MAPEM.idx; j < i + GeoPar.nelem; j += MAPEM.idxmax)
			{
				p = MAPEM.Wjj[j] / (REAL) 9.0 - recon[j]
						+ MAPEM.Sx[j] * MAPEM.Sjjinv[j] / (REAL) 9.0;
				q = MAPEM.qj[j] / (REAL) 9.0;
				if ((p <= 0.0) || (q / (p * p) >= 0.005))
				{
					MAPEM.qj[j] = (REAL) 0.5
							* (-p + (REAL) sqrt(p * p + 4 * q));
				}
				else
				{
					MAPEM.qj[j] = q / p * (1 - (REAL) 0.5 * (q / (p * p)));
				}
			}
		}
	}    // not with the blob case .hstau

	//====== step 4: replace k-th iteration by (k+1)-th iteration in the
	//       reconstruction
	if (Blob.pix_basis)
	{
		for (i = MAPEM.idy * GeoPar.nelem;
				i < GeoPar.nelem * (GeoPar.nelem - 1) + 1;
				i += GeoPar.nelem * MAPEM.idymax)
		{
			for (j = i + MAPEM.idx; j < i + GeoPar.nelem; j += MAPEM.idxmax)
			{
				if (MAPEM.qj[j] < Consts.zero)
				{
					recon[j] = Consts.zero;
				}
				else
				{
					recon[j] = MAPEM.qj[j];
				}
			}
		}
	}

	else
	{  //if blobs
		for (i = MAPEM.idy * Blob.M; i < Blob.M * (Blob.H - 1) + 1;
				i += Blob.M * MAPEM.idymax)
		{
			for (j = i + MAPEM.idx; j < i + Blob.M; j += MAPEM.idxmax)
			{

				if (j % 2 == Blob.pr)
				{ //only the hex grid points
					if (MAPEM.qj[j] < Consts.zero)
					{
						recon[j] = Consts.zero;
					}
					else
					{
						recon[j] = MAPEM.qj[j];
					}
				}
			}
		}
	}

	return FALSE;
}

/*      
 c======================================================================
 c     source & dest are nelem*nelem arrays.  source contains a picture
 c     to be convolved with the 3*3 kernel whose central element is 1
 c     and the rest are -1/8.  The result goes in dest.
 c======================================================================
 */

void emap_class::applyK(REAL* source, REAL* dest, INTEGER nelem)
{
	INTEGER i, j;

	for (i = 0; i < nelem * nelem; i++)
	{
		dest[i] = 0.0;
	}

	for (i = 0; i < nelem - 1; i++)
	{
		for (j = 0; j < nelem; j++)
		{
			dest[i + (j) * nelem] += source[i + (j) * nelem + 1];

		}
	}

	for (i = 0; i < nelem - 1; i++)
	{
		for (j = 0; j < nelem; j++)
		{
			dest[i + (j) * nelem + 1] += source[i + (j) * nelem];

		}
	}

	for (i = 0; i < nelem; i++)
	{
		for (j = 0; j < nelem - 1; j++)
		{
			dest[i + (j) * nelem] += source[i + (j) * nelem + nelem];
		}
	}

	for (i = 0; i < nelem; i++)
	{
		for (j = 0; j < nelem - 1; j++)
		{

			dest[i + (j) * nelem + nelem] += source[i + (j) * nelem];

		}
	}

	for (i = 0; i < nelem - 1; i++)
	{
		for (j = 0; j < nelem - 1; j++)
		{
			dest[i + (j) * nelem] += source[i + (j) * nelem + nelem + 1];
		}
	}

	for (i = 0; i < nelem - 1; i++)
	{
		for (j = 0; j < nelem - 1; j++)
		{
			dest[i + (j) * nelem + nelem + 1] += source[i + (j) * nelem];

		}
	}

	for (i = 0; i < nelem - 1; i++)
	{
		for (j = 0; j < nelem - 1; j++)
		{
			dest[i + (j) * nelem + nelem] += source[i + (j) * nelem + 1];

		}
	}

	for (i = 0; i < nelem - 1; i++)
	{
		for (j = 0; j < nelem - 1; j++)
		{
			dest[i + (j) * nelem + 1] += source[i + (j) * nelem + nelem];

		}
	}

	sscal(nelem * nelem, -1 / 8., dest, 1);

	for (i = 0; i < nelem * nelem; i++)
	{
		dest[i] += source[i];

	}

	return;
}
