/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/foru_frinit.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 PURPOSE: INITIALIZE ALL GLOBAL VARIABLES AND ALLOCATE SPACE
 IMPORTANT VARIABLES:
 RINC  MODIFIED PINC,FOR UNI RINC=PINC
 FOR VRI RINC=PINC*RADIUS/STOD
 DELD  SAMPLE SPACING ON TRANSFORMED PROJECTIONS
 DELP  SAMPLE SPACING ON FOURIER PLAN
 RADMAX  CUTOFF RADIUS
 MIDR  NSIZE1/2
 MIDP  NSIZE2/2
 EPSLN  A SMALL NUMBER TO HANDLE ROUNDING ERROR
 NFORW(3)  3 ELEMENTS ARRAY TO INDICATE THE DIMENSION
 OF THE ARRAY TO BE FORWARD TRANSFORMED
 NBACK(3)  3 ELEMENTS ARRAY TO INDICATE THE DIMENSION
 OF THE ARRAY TO BE BACKTRANSFORMED
 NPROJ1,NPROJ2  STARTING ADDRESSES OF ARRAYS TO HOLD
 TRANSFORMED PROJECTIONS
 NFRPLN  STARTING ADDRESS OF THE ARRAY TO REPRESENT
 FOURIER PLANE
 IFLG  INITIALLY 0, AND SET TO 1 IF ERROR OCCURS
 RESERV  RESERVE FOR FUTURE USE
 */

#include <cstdio>

#include "blkdta.h"
#include "geom.h"
#include "fourie.h"
#include "consts.h"
#include "uiod.h"
#include "anglst.h"
#include "infile.h"

#include "foru.h"

void foru_class::frinit()
{
	INTEGER filter;
	BOOLEAN eol;
	INTEGER size1;
	INTEGER size2;
	INTEGER word;
	INTEGER k3;
	INTEGER k4;
	INTEGER k;
	INTEGER kh;
	REAL angfst;
	REAL sinth;
	REAL costh;
	REAL anglst;
	REAL angspn;


	//data fname/4hband,4hsinc,4hcosi,4hhamm/
	// modified calls to getwrd() to use the 4-parameter version. Lajos, Dec 13, 2004
	static const INTEGER fname[4] =
	{
			CHAR2INT('b', 'a', 'n', 'd'), CHAR2INT('s', 'i', 'n', 'c'),
			CHAR2INT('c', 'o', 's', 'i'), CHAR2INT('h', 'a', 'm', 'm')
	};

	//data samp/4hsamp/,pict/4hpict/
	static const INTEGER samp = CHAR2INT('s', 'a', 'm', 'p');
	static const INTEGER pict = CHAR2INT('p', 'i', 'c', 't');

	// modified calls to getwrd() to use the 4-parameter version. Lajos, Dec 13, 2004
	static const INTEGER foru_codes[2] =
	{
	CHAR2INT('s', 'a', 'm', 'p'), CHAR2INT('p', 'i', 'c', 't') };

	Fourie.iflg = 0;

	fprintf(output, "\nf o u r i e r  r e c o n s t r u c t i o n      \n");

	if (GeoPar.par)
		Fourie.rinc = GeoPar.pinc;
	if (GeoPar.div)
		Fourie.rinc = GeoPar.pinc * GeoPar.radius / GeoPar.stod;
	if (GeoPar.strip)
	{
		fprintf(output, "\nstrip projections");

	}
	if (GeoPar.div)
	{
		fprintf(output, "\ndivergent rays, will be treated as parallel");
	}
	filter = InFile.getwrd(TRUE, &eol, fname, 4);
	if (eol)
		error(5);

	for (Fourie.nfiltr = 0; Fourie.nfiltr < 4; Fourie.nfiltr++)
	{
		if (filter == fname[Fourie.nfiltr])
			goto L25;
	}
	error(6);

	L25: Fourie.cutoff = InFile.getnum(FALSE, &eol);
	if (eol)
		error(5);
	// bug 151 - swr - 9/24/05
	if (Fourie.cutoff < 0.0)
		Fourie.cutoff = (REAL) 2.0 * GeoPar.prjnum * Fourie.rinc
				/ ((REAL) Consts.pi * GeoPar.nelem * GeoPar.pixsiz);
	if (Fourie.cutoff <= Consts.zero)
		Fourie.cutoff = 1.0;
	if (Fourie.cutoff > 1.0)
		Fourie.cutoff = 1.0;
	Fourie.interp = InFile.getint(FALSE, &eol);
	if (eol)
		error(5);
	if ((Fourie.interp > 4) || (Fourie.interp < 1))
		error(8);
	size1 = GeoPar.nrays + 1;
	size2 = GeoPar.nelem + 1;
	word = InFile.getwrd(FALSE, &eol, foru_codes, 2);
	if (!eol)
	{

		if (word == pict)
			goto L4;
		if (word != samp)
			error(6);
		size1 = InFile.getint(FALSE, &eol);
		if (eol)
			error(5);
		if (eol)
			return;
		word = InFile.getwrd(FALSE, &eol, &(foru_codes[1]), 1);

		if (!eol)
		{
			if (word == pict)
				goto L4;
			error(6);
			return;

			L4: size2 = InFile.getint(FALSE, &eol);
			if (eol)
				error(5);
			if (eol)
				return;
		}
	}

	if (size1 <= GeoPar.usrays)
		size1 = GeoPar.usrays + 1;
	if (size1 != (size1 / 2 * 2))
		size1++;
	Fourie.nsize1 = size1;
	if (size2 <= GeoPar.nelem)
		size2 = GeoPar.nelem + 1;
	if (size2 != (size2 / 2 * 2))
		size2++;
	Fourie.nsize2 = size2;
	if (Fourie.iflg == 1)
		return;

	fprintf(output, "\nfilter=   %2i", Fourie.nfiltr);
	fprintf(output, "\ncutoff=    %5.2f", Fourie.cutoff);
	fprintf(output, "\ninterp=     %2i", Fourie.interp);
	fprintf(output, "\nnsize1=    %4i", Fourie.nsize1);
	fprintf(output, "\nnsize2=   %4i", Fourie.nsize2);

	Fourie.radmax = (Fourie.nsize1 / 2) * Fourie.cutoff;
	if (Fourie.nfiltr == 4)
		Fourie.radmax = (REAL) Fourie.nsize1 / 2;
	Fourie.midr = Fourie.nsize1 / 2;
	Fourie.midp = Fourie.nsize2 / 2;
	Fourie.nforw[0] = 1;
	Fourie.nforw[1] = Fourie.nsize1 / 2;
	Fourie.nforw[2] = 1;
	Fourie.nback[0] = Fourie.nsize2;
	Fourie.nback[1] = Fourie.nsize2 / 2;
	Fourie.nback[2] = 1;

	if (Fourie.ntemp != NULL)
		delete[] Fourie.ntemp;
	if (Fourie.nproj1 != NULL)
		delete[] Fourie.nproj1;
	if (Fourie.nproj2 != NULL)
		delete[] Fourie.nproj2;
	Fourie.ntemp = new REAL[Fourie.nsize1];
	Fourie.nproj1 = new REAL[Fourie.nsize1 + 4];
	Fourie.nproj2 = new REAL[Fourie.nsize1 + 4];

	k3 = Fourie.nsize1 + 2;
	k4 = Fourie.nsize1 + 4;

	for (k = k3; k < k4; k++)
	{
		Fourie.nproj1[k] = 0.0;
		Fourie.nproj2[k] = 0.0;
	}
	if (Fourie.nfrpln != NULL)
		delete[] Fourie.nfrpln;
	Fourie.nfrpln = new REAL[Fourie.nsize2 * (Fourie.nsize2 + 2)];
	kh = Fourie.nsize2 * (Fourie.nsize2 + 2);
	for (k = 0; k < kh; k++)
	{
		Fourie.nfrpln[k] = 0.0;
	}

	Fourie.deld = (REAL) 1.0 / (Fourie.nsize1 * Fourie.rinc);
	Fourie.delp = (REAL) 1.0 / (Fourie.nsize2 * GeoPar.pixsiz);
	Anglst.getang(0, &angfst, &sinth, &costh);
	Anglst.getang(GeoPar.prjnum - 1, &anglst, &sinth, &costh);
	angspn = anglst - angfst;
	Fourie.epsln = (REAL) 1.0e-3;
	if (GeoPar.par && GeoPar.uni
			&& ((angspn < Consts.pid2) || (angspn >= Consts.pi)))
		error(1);
	if (GeoPar.par && GeoPar.vri
			&& ((angspn < (0.75 * Consts.pi)) || (angspn >= Consts.pi)))
		error(1);
	if (GeoPar.div
			&& ((angspn < (1.5 * Consts.pi)) || (angspn >= Consts.twopi)))
		error(2);

	return;
}
