/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/mart.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 IMPLEMENTATIONS OF VARIOUS VERSIONS OF MULTIPLICATIVE ART ALGORITHM
 PROGRAMMED BY A. V. LAKSHMINARAYANAN AND A. LENT
 SUNY AT BUFFALO, MAY, 1977;  REVISED JAN 78
 IT IS ASSUMED THAT PROJECTION DATA ARE INTRINSICALLY NONNEGATIVE
 NONNEGATIVE RECONSTRUCTIONS ARE SOUGHT

 DESCRIPTION OF PARAMETERS

 INPUT CONSISTS OF THE KEYWORD 'METHOD' FOLLOWED BY ONE OF FOUR
 OPTIONS 'GBH', 'LENT', 'LAK1', 'LAK2', FOLLOWED BY AN INTEGER
 MODIFIER KOUNT, TWO FLOATING POINT MODIFIERS  RELAX AND DATZRO,
 OPTIONALLY FOLLOWED BY WORD MODIFIERS 'NORMAL', 'ENTROPY'
 THE KEYWORD 'METHOD' MUST BE PRESENT; ITS MODIFIERS HAVE THE
 FOLLOWING MEANING

 'GBH', USE MULTIPLICATIVE ART ALGORITHM OF GORDON AND HERMAN
 INT. REV. CYTOL. 38, 1974
 START WITH PICTURE UNIFORMLY AVEDEN
 RELAXATION PARAMETER ADDED TO LITERATURE VERSION OF
 ALGORITHM
 'LENT', USE MULTIPLICATIVE ART ALGORITHM OF LENT
 PROC. CONF ON IMAGE ANALYSIS AND EVALUATION, JULY,1976, TORON
 SOCIETY OF PHOTOGRAPHIC SCIENTISTS AND ENGINEERS
 START WITH PICTURE AVEDEN EXCEPT FOR PIXELS IN RAYS WITH ZERO
 PROJECTION DATA
 THIS STARTING POINT IS IMMEDIATELY CHANGED BY APPLICATION
 OF NORMALIZATION CONSTRAINT
 NORMALIZE COEFFICIENT MATRIX BY ROWS

 'LAK1', USE (APPROX) MULTIPLICATIVE ART ALGORITHM OF LAKSHMINARAYANAN
 LINEAR APPROXIMATION TO THE NONINTEGRAL POWER OF MULT ART.
 FIRST TWO TERMS OF THE BINOMIAL EXPANSION USED.
 USE SAME STARTING POINT AS UNDER 'LENT'
 NORMALIZE COEFFICIENTS BY MAX ELEMENT OF PROJECTION MATRIX

 'LAK2', USE (APPROX) MULT ART OF LAKSHMINARAYANAN
 QUADRATIC APPROXIMATION TO THE NONINTEGRAL POWER OF MULT ART.
 FIRST THREE TERMS OF THE BINOMIAL EXPANSION USED.
 SAME STARTING POINT AND COEFFICIENT NORMALIZATION AS 'LAK1'

 IF KOUNT .NE. 0, USE ONLY THE FIRST KOUNT RAYS OF THE PICK SEQUENCE
 THE EFFECT OF KOUNT IS DETERMINED BY THE SNARK SELECT CARD
 DEFAULT...IF KOUNT .LE. 0, KOUNT RESET  TO COMPLETE SET OF PROJ DATA

 RELAX IS THE UNDERRELXATION PARAMETER.
 VALUES LESS THAN 1.0 ARE SUGGESTED FOR NOISY PROJECTION DATA.
 VALUES BIGGER THAN 1.0 ARE NOT PROHIBITED BY THEORY AND MAY SPEED
 CONVERGENCE.
 THE    PRECEDING REMARKS ARE NOT MEANINGFUL FOR 'GBH'
 FOR 'GBH', RELAX MUST BE CONSIDERED TO HAVE DIMENSIONS

 DATZRO..ANY PROJECTION DATA WITH VALUE .LE. THAN DATZRO WILL BE
 CONSIDERED ZERO BY THE PROGRAM.

 'NORMAL' , IF PRESENT, NORMALIZE RECON TO DENSITY AVEDEN
 AT END OF EACH ITERATION

 'ENTROPY' ,IF PRESENT, CALCULATE AND PRINT ENTROPY FUNCTIONAL
 DEFINITION
 LET T = SIGMA RECON(I) OVER ALL PIXELS
 LET Y(I) = RECON(I)/T
 ENTROPY = - SIGMA Y(I)*ALOG(Y(I))

 EXAMPLE INPUT SEQUENCES FOLLOW
 METHOD GBH 0 RELAX = 1.0 IGNORE RAYSUMS LT 0.0
 METHOD LAK1 KOUNT = 0  0.8  0.0 ENTROPY
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "uiod.h"
#include "int2str.h"
#include "pick.h"
#include "ray.h"
#include "wray.h"
#include "anglst.h"
#include "pseudo.h"
#include "infile.h"
#include "blob.h"
#include "raysel.h"

#include "projfile.h"  //for debugg purpose

#include "mart.h"

BOOLEAN mart_class::Run(REAL* recon, INTEGER* list, REAL* weight, INTEGER iter)
{
	static const INTEGER hmeth = CHAR2INT('m', 'e', 't', 'h');
	static const INTEGER hnorm = CHAR2INT('n', 'o', 'r', 'm');
	static const INTEGER hentr = CHAR2INT('e', 'n', 't', 'r');

	static const INTEGER gbh = CHAR2INT('g', 'b', 'h', ' ');
	static const INTEGER lent = CHAR2INT('l', 'e', 'n', 't');
	static const INTEGER lak1 = CHAR2INT('l', 'a', 'k', '1');
	static const INTEGER lak2 = CHAR2INT('l', 'a', 'k', '2');

	BOOLEAN xalg0_tmp;
	BOOLEAN eol;
	static INTEGER nalg;
	INTEGER word;
	static REAL datzro; //set static
	static INTEGER lnorm;
	static INTEGER lfnal;
	INTEGER i;
	INTEGER np;
	INTEGER nr;
	REAL raysum;
	INTEGER numb;
	REAL snorm;
	INTEGER j;
	INTEGER l;
	INTEGER isum;
	REAL fact;
	static REAL aijmax; // set static
	REAL wj;
	INTEGER k;
	REAL trysum;
	REAL corfac;
	REAL delu;
	REAL ajmax;
	REAL corr;
	REAL roaij;
	REAL reldif;
	REAL aij;
	REAL hafrel;
	REAL hafabs;
	REAL fac;
	REAL tot;
	REAL entrop;
	REAL rot;
	static INTEGER kount;
	static REAL relax; //set static

	INTEGER area;

	if (Blob.pix_basis)
	{
		area = GeoPar.area;
		//    num_rays = GeoPar.nrays;
	}

	else
	{
		area = Blob.area;
		//    num_rays = GeoPar.usrays;
	}

	xalg0_tmp = FALSE;

	if (!(iter > 1))
	{

		// GET THE METHOD

		INTEGER exp0[1] =
		{ hmeth };
		word = InFile.getwrd(TRUE, &eol, exp0, 1);

		if (eol)
		{
			fprintf(output,
					"\n          **** error - keyword method is missing***");
			return TRUE;
		}

		INTEGER exp1[4] =
		{ gbh, lent, lak1, lak2 };

		word = InFile.getwrd(FALSE, &eol, exp1, 4);

		if (eol)
		{
			fprintf(output, "\n          **** error - one and only one of the following methods must be specified : gbh, lent, lak1, or lak2***");
			return TRUE;
		}

		if (word == gbh)
		{
			nalg = 1;
			fprintf(output, "\n          multiplicative art version %s",
					int2str(gbh));
		}
		else if (word == lent)
		{
			nalg = 2;
			fprintf(output, "\n          multiplicative art version %s",
					int2str(lent));
		}
		else if (word == lak1)
		{
			nalg = 3;
			fprintf(output, "\n          multiplicative art version %s",
					int2str(lak1));
		}
		else
		{  //lak2
			nalg = 4;
			fprintf(output, "\n          multiplicative art version %s",
					int2str(lak2));
		}

		// GET KOUNT, RELAXATION PARAMETER, RAYSUM LIMIT, NORMAL, ENTROPY
		kount = InFile.getint(FALSE, &eol);

		relax = InFile.getnum(FALSE, &eol);

		datzro = InFile.getnum(FALSE, &eol);

		if (eol)
		{
			fprintf(output,
					"\n          **** some data is missing: either kount, relax, or data_zero***");
			return TRUE;
		}

		// optional inputs

		INTEGER exp3[2] =
		{ hnorm, hentr };
		word = InFile.getwrd(FALSE, &eol, exp3, 2);

		if (eol)
			goto L10;
		// finished reading input

		lnorm = FALSE;
		lfnal = FALSE;

		if (word == hnorm)
		{
			lnorm = TRUE;
			INTEGER exp3a[1] =
			{ hentr };
			word = InFile.getwrd(FALSE, &eol, exp3a, 1);
			if (word == hentr)
				goto L5;
			else
				goto L10;
		}

		else
		{
			L5: lfnal = TRUE;
		}

		//***  CHECKING INPUTS AND SETTING DEFAULTS
		L10: if (kount <= 0)
		{
			if (RaySel.useray)  // select user
				kount = GeoPar.prjnum * GeoPar.usrays; // change nrays to usrays. hstau 7/2003
			else
				//select snark
				kount = GeoPar.prjnum * GeoPar.snrays;
		} // set value to kount for select user or select snark. Wei   11/2004

		if (relax <= 0)
			relax = 1.0;
		if ((nalg == 3) && (relax > 1.0))
			relax = 1.0;
		if (datzro < Consts.zero)
			datzro = Consts.zero;

		fprintf(output, "\n          no of rays used in each iteration %10i",
				kount);
		fprintf(output, "\n          underrelaxation factor %10.5f", relax);
		fprintf(output,
				"\n          all projection data with values less than %20.10e are ignored",
				datzro);

		if (lnorm)
		{
			fprintf(output, "\n          reconstruction is normalized");
		}
		else
		{
			fprintf(output, "\n          reconstruction not normalized");
		}

		if (lfnal)
		{
			fprintf(output, "\n          entropy functional is calculated");
		}
		else
		{
			fprintf(output, "\n          entropy functional not calculated");
		}

		if (GeoPar.aveden <= Consts.zero)
		{
			fprintf(output,
					"\n ***aveden too small.  probably incorrect use of mart, no execution***");
			return TRUE;
		}

		for (i = 0; i < area; i++)
		{
			recon[i] = GeoPar.aveden;
		}

		//***  NO CHOICE IN INITIAL GUESS; EXALG OPTIONS DONT OPERATE

		if (nalg != 1)
		{
			//  WILL ZERO OUT ANY PIXEL IN A RAY WITH ZERO PROJ DATA
			for (np = 0; np < GeoPar.prjnum; np++)
			{
				for (nr = 0; nr < GeoPar.nrays; nr++)
				{
					raysum = Anglst.prdta(np, nr);
					if (raysum <= datzro)
					{
						if (GeoPar.strip)
						{
							if (Blob.pix_basis)
							{
								ray(np, nr, list, weight, &numb, &snorm);
							}
							else
							{
								Blob.bray(np, nr, list, weight, &numb, &snorm);
							}
						}
						if (GeoPar.line)
						{
							if (Blob.pix_basis)
							{
								wray(np, nr, list, weight, &numb, &snorm);
							}
							else
							{
								Blob.bwray(np, nr, list, weight, &numb, &snorm);
							}
						}
						if (numb != 0)
						{
							for (j = 0; j < numb; j++)
							{
								l = list[j];
								recon[l] = 0.0;
							}
						}
					}
				}

			}

			isum = 0;

			for (i = 0; i < area; i++)
			{
				if (recon[i] != 0.0)
				{

					if (!Blob.pix_basis)
					{ //if blobs
						if (i % 2 == Blob.pr)
						{
							isum++;
						}
					}
					else
					{
						isum++;
					}
				}
			}

			if (isum == 0)
			{

				fprintf(output,
						"\n  ***warning-  all projection data near zero. zero reconstruction.***");
				return TRUE;
			}

			//  NORMALIZE

			fact = area * GeoPar.aveden / isum;
			if (!Blob.pix_basis)
			{ //if blobs
				fact = fact / 2.0;
			}
			for (i = 0; i < area; i++)
			{
				if (recon[i] != 0.0)
					recon[i] = fact;
			}

			//  LOCATE BIGGEST ELEMENT IN PROJECTION MATRIX FOR LAK1 AND LAK2 ONLY
			//  THIS STRATEGY IS CONSERVATIVE IN THAT COEFFICIENTS OF PIXELS
			//  WHOSE VALUES ARE KNOWN TO BE ZERO ARE INCLUDED IN THE SEARCH
			if (nalg != 2)
			{
				aijmax = 0.0;
				for (np = 0; np < GeoPar.prjnum; np++)
				{
					for (nr = 0; nr < GeoPar.nrays; nr++)
					{
						if (GeoPar.strip)
						{
							if (Blob.pix_basis)
							{
								ray(np, nr, list, weight, &numb, &snorm);
							}
							else
							{
								Blob.bray(np, nr, list, weight, &numb, &snorm);
							}
						}
						if (GeoPar.line)
						{
							if (Blob.pix_basis)
							{
								wray(np, nr, list, weight, &numb, &snorm);
							}
							else
							{
								Blob.bwray(np, nr, list, weight, &numb, &snorm);
							}
						}
						if (numb != 0)
						{
							for (j = 0; j < numb; j++)
							{
								wj = weight[j];
								if (wj > aijmax)
									aijmax = wj;
							}
						}
					}

				}

				//  BASIC UPDATE PART OF PROGRAM
			}
		}
	}

	switch (nalg)
	{

	//  GORDON,BENDER,HERMAN MULT ART
	case 1:
		for (k = 0; k < kount; k++)
		{
			pick(&np, &nr);

			raysum = Anglst.prdta(np, nr);
			if (Blob.pix_basis)
			{
				trysum = pseudo(recon, np, nr, list, weight, &numb, &snorm,
						GeoPar.line, FALSE);
			}
			else
			{
				trysum = Blob.bpseudo(recon, np, nr, list, weight, &numb,
						&snorm, GeoPar.line, FALSE);
			}
			if (numb != 0)
			{
				if (raysum <= datzro)
				{
					for (j = 0; j < numb; j++)
					{
						l = list[j];
						recon[l] = 0.0;
					}
				}
				else
				{

					if ((trysum <= Consts.zero) && (trace >= 1))
					{
						fprintf(output,
								"\n zero pseudo proj dat %5i %5i %10.5f %10.5f",
								np, nr, raysum, trysum);
					}

					if (trysum > Consts.zero)
					{
						corfac = relax * (REAL) log(raysum / trysum);
						for (j = 0; j < numb; j++)
						{
							l = list[j];
							recon[l] *= (REAL) exp(corfac * weight[j]);
						}
					}
				}
			}
		}

		//  END GBH UBDATE
		break;

		//  LENT MULT ART

	case 2:

		for (k = 0; k < kount; k++)
		{
			pick(&np, &nr);
			raysum = Anglst.prdta(np, nr);
			if (raysum > datzro)
			{
				if (Blob.pix_basis)
				{
					trysum = pseudo(recon, np, nr, list, weight, &numb, &snorm,
							GeoPar.line, FALSE);
				}
				else
				{
					trysum = Blob.bpseudo(recon, np, nr, list, weight, &numb,
							&snorm, GeoPar.line, FALSE);
				}
				if (numb != 0)
				{
					if (trysum != 0.0)
					{
						delu = (REAL) log(raysum / trysum);

						// FIND ROW SCALE FACTOR
						ajmax = 0.0;

						for (j = 0; j < numb; j++)
						{
							if (weight[j] > ajmax)
								ajmax = weight[j];
						}

						corr = relax * delu / ajmax;

						for (j = 0; j < numb; j++)
						{
							l = list[j];
							recon[l] *= (REAL) exp(corr * weight[j]);
						}

					}
				}
			}
		}

		// END LENT UPDATE
		break;

		// LAK APPROX MULT ART - LINEAR APPROXIMATION
	case 3:

		roaij = relax / aijmax;

		for (k = 0; k < kount; k++)
		{
			pick(&np, &nr);
			raysum = Anglst.prdta(np, nr);
			if (raysum > datzro)
			{
				if (Blob.pix_basis)
				{
					trysum = pseudo(recon, np, nr, list, weight, &numb, &snorm,
							GeoPar.line, FALSE);
				}
				else
				{
					trysum = Blob.bpseudo(recon, np, nr, list, weight, &numb,
							&snorm, GeoPar.line, FALSE);
				}
				if (!((numb == 0) || (trysum <= Consts.zero)))
				{
					reldif = (raysum - trysum) / MAX0(raysum, trysum);

					// LINEAR APPROX
					for (j = 0; j < numb; j++)
					{
						l = list[j];
						aij = weight[j] * roaij;
						recon[l] *= ((REAL) 1.0 + reldif * aij);
					}
				}
			}
		}
		break;

		//END LAK1 UPDATE
		// LAK APPROX MULT ART - QUADRATIC APPROXIMATION

	case 4:

		roaij = relax / aijmax;

		for (k = 0; k < kount; k++)
		{
			pick(&np, &nr);
			raysum = Anglst.prdta(np, nr);
			if (raysum > datzro)
			{
				if (Blob.pix_basis)
				{
					trysum = pseudo(recon, np, nr, list, weight, &numb, &snorm,
							GeoPar.line, FALSE);
				}
				else
				{
					trysum = Blob.bpseudo(recon, np, nr, list, weight, &numb,
							&snorm, GeoPar.line, FALSE);
				}
				if (!((numb == 0) || (trysum <= Consts.zero)))
				{
					reldif = (raysum - trysum) / MAX0(raysum, trysum);

					// QUADRATIC APPROX
					hafrel = (REAL) 0.5 * reldif;
					hafabs = (REAL) fabs(hafrel);
					for (j = 0; j < numb; j++)
					{
						l = list[j];
						aij = weight[j] * roaij;
						fac = (REAL) 1.0
								+ reldif * aij
										* ((REAL) 1.0 + hafrel * aij + hafabs);
						recon[l] *= fac;
					}
				}
			}
		}
		break;

	}
	//  END LAK2 UPDATE

	//  CHECK OPTIONS

	if (!(lnorm || lfnal))
		return xalg0_tmp;

	tot = 0.0;

	for (i = 0; i < area; i++)
	{
		if (!Blob.pix_basis)
		{ //if blobs
			if (i % 2 == Blob.pr)
			{
				tot += recon[i];
			}
		}
		else
		{
			tot += recon[i];
		}
	}

	if (tot > Consts.zero)
	{
		if (lnorm)
		{
			fact = GeoPar.aveden * area / tot;  // aveden in both basis concide
			if (!Blob.pix_basis)
			{
				fact /= 2.0;
			}
			for (i = 0; i < area; i++)
			{
				recon[i] *= fact;
			}

			tot = GeoPar.aveden * area;
			if (!Blob.pix_basis)
			{
				tot /= 2.0;
			}
		}

		if (!lfnal)
			return xalg0_tmp;

		entrop = 0.0;
		for (i = 0; i < area; i++)
		{
			if (recon[i] > 0.0)
			{
				rot = recon[i] / tot;
				entrop -= rot * (REAL) log(rot); //remove the minus sign here, wei
			}
		}

		fprintf(output,
				"\n                                                        iter %5i entropy %20.10e",
				iter, entrop);

		return xalg0_tmp;
	}
	fprintf(output,
			"\n ***total density %20.10e  is too small, no normalization or entropy calculation***",
			tot);
	return xalg0_tmp;
}
