/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/qintp.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 INITIALIZE CONVOLUTING FUNCTION FOR CONVOLUTION ALGORITHM.
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "consts.h"

#include "qintp.h"

REAL qintp(REAL pos, REAL* table, INTEGER n, INTEGER interp)
{
	INTEGER epos;
	INTEGER opos;
	REAL er;
	REAL or1;
	INTEGER i;

	REAL a0;
	REAL a1;
	REAL a2;
	REAL a3;
	REAL sum;
	REAL sgn;
	INTEGER k;
	REAL a1t2;
	REAL a2t2;
	REAL a4;
	REAL a0t2;
	REAL a1t4;
	REAL a2t4;
	REAL a3t4;
	REAL a4t2;
	REAL p1pm2;
	REAL p2pm3;
	REAL ppm1;
	REAL pa23;
	REAL pa14;
	REAL pa05;

	epos = (INTEGER) pos;
	opos = (INTEGER) (pos + 0.5);
	er = pos - epos;
	or1 = pos - opos;
	i = interp + 2;

	/// fix for indexes in C++
	epos--;
	opos--;

	switch (i)
	{

	// MODIFIED CUBIC SPLINE

	case 1:
		a0 = table[epos - 1];
		a1 = table[epos];
		a2 = table[epos + 1];
		a3 = table[epos + 2];
		return a1 + (REAL) 0.5 * er * (a2 - a0 + er	* ((REAL) 2.0 * a0 - (REAL) 5.0 * a1 + (REAL) 4.0 * a2 - a3	+ er * (a3 - a0 + (REAL) 3.0 * (a1 - a2))));

		// BAND LIMITING (SINC) INTERPOLATION

	case 2:
		if (fabs(or1) < Consts.zero)
		{
			return table[opos];
		}
		sum = 0.0;
		sgn = -1.0;
		for (k = 0; k < n; k++)
		{
			sum += sgn * table[k] / (pos - k);
			sgn = -sgn;
		}

		if (((opos / 2) * 2) == opos)
		{
			return sum * (REAL) sin(Consts.pi * or1) / Consts.pi;
		}
		else
		{
			return -sum * (REAL) sin(Consts.pi * or1) / Consts.pi;
		}

		// NEAREST POINT

	case 3:
		return table[opos];

		// LINEAR INTERPOLATION

	case 4:
		return table[epos] + er * (table[epos + 1] - table[epos]);

		// 3 POINT LAGRANGE

	case 5:
		a0 = table[opos - 1] / (REAL) 2.0;
		a1 = table[opos];
		a2 = table[opos + 1] / (REAL) 2.0;
		return a1 + or1 * (-a0 + a2 + or1 * (a0 - a1 + a2));

		// 4 POINT LAGRANGE

	case 6:
		a0 = table[epos - 1] / (REAL) 6.0;
		a1 = table[epos] / (REAL) 2.0;
		a2 = table[epos + 1] / (REAL) 2.0;
		a3 = table[epos + 2] / (REAL) 6.0;
		a1t2 = table[epos];
		a2t2 = table[epos + 1];
		return a1t2 + er * ((REAL) -2.0 * a0 - a1 + a2t2 - a3 + er * ((REAL) +3.0 * a0 - a1t2 + a2 + er * (-a0 + a1 - a2 + a3)));

		// 5 POINT LAGRANGE

	case 7:
		a0 = table[opos - 2] / (REAL) 24.0;
		a1 = table[opos - 1] / (REAL) 6.0;
		a2 = table[opos] / (REAL) 4.0;
		a3 = table[opos + 1] / (REAL) 6.0;
		a4 = table[opos + 2] / (REAL) 24.0;
		a0t2 = (REAL) 2.0 * a0;
		a1t4 = (REAL) 4.0 * a1;
		a2t4 = table[opos];
		a3t4 = (REAL) 4.0 * a3;
		a4t2 = (REAL) 2.0 * a4;
		return a2t4 + or1 * (a0t2 - a1t4 + a3t4 - a4t2 + or1 * (-a0 + a1t4 - (REAL) 5.0 * a2 + a3t4 - a4 + or1 * (-a0t2 + a1 - a3 + a4t2 + or1 * (a0 - a1 + a2 - a3 + a4))));

		// 6 POINT LAGRANGE

	case 8:
		p1pm2 = (er + (REAL) 1.0) * (er - (REAL) 2.0);
		p2pm3 = (er + (REAL) 2.0) * (er - (REAL) 3.0);
		ppm1 = er * (er - (REAL) 1.0);
		pa23 = p1pm2 * p2pm3 / (REAL) 12.0;
		pa14 = p2pm3 * ppm1 / (REAL) 24.0;
		pa05 = ppm1 * p1pm2 / (REAL) 120.0;
		return -pa05 * (er - (REAL) 3.0) * table[epos - 2]
				+ pa14 * (er - (REAL) 2.0) * table[epos - 1]
				- pa23 * (er - (REAL) 1.0) * table[epos]
				+ pa23 * er * table[epos + 1]
				- pa14 * (er + (REAL) 1.0) * table[epos + 2]
				+ pa05 * (er + (REAL) 2.0) * table[epos + 3];

	default:
		// error invalid parameter interp
		return 0;
	}
}
