/** @file line_window_onevar_t.hpp
    @author Deniz
    @description class line_window_onevar_t is defined
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#ifndef _LINE_WINDOW_ONEVAR_T
#define _LINE_WINDOW_ONEVAR_T

#include <list>
#include <qstring.h>

#include <qcanvas.h>
class QColor;
#include "line_window_t.hpp"
class line_real_t;
typedef std::list<line_window_t> data_onevar_t;

/* This corresponds to a set of maybe disconnected (due to falling out of window boundaries)
   segments for a single variable, due to single execution
   All these hacks are due to a bug i found in QT API (both versions)
   when plotting big numbers: now being superparanoid to drawLine() ONLY within the window, 
   which is not necessarily as precise but at least guarantees that 
   there will be lines when there should be. 
*/
class line_window_onevar_t
{
  data_onevar_t data;
  QString plotname;
  friend class plot_t;
public:
  /** constuctor/1 takes one line with real coords, potentially makes many point sets from it    *
   *  pre: line_real has at least 1 point                                                        *
   *  pre: x values in line_real are strictly increasing (as is the case in snark14 eval output) *
   *  (as well as in column/row profiles?) i believe those are not even given as (x,y)  (yet)    *
   *  trusts Painter::drawLine() with between interior and JUST outside the paint window.        *
   */
  line_window_onevar_t(line_real_t& line_real);
  bool empty();
  QString getName();
  void plot(QPainter& P, QColor& C, Qt::PenStyle & Ps);
  void show();
}; // -- class line_window_onevar_t
#endif
