/** @file line_window_t.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include <iostream>

#include <qpainter.h>
#include <qcolor.h>

#include "sd_line_t.hpp"
#include "line_window_t.hpp"

void line_window_t::add(int x, int y) {
  point_window_t mypoint = {x, y};
  data.push_back(mypoint);
} // --line_window_t::add/2

void line_window_t::add(point_window_t& ptw) {
  data.push_back(ptw);
} // --line_window_t::add/1

bool line_window_t::empty() { return data.empty(); } // --line_window_t::empty()

void line_window_t::clear() { data.clear(); } // --line_window_t::clear()

/** pre: all coordinates are within paint's win          */
/* (i.e., all points in data ought to be within window.) */
void line_window_t::plot(QPainter& P, QColor& C, Qt::PenStyle & Ps) {
  // modify P somehow: just a point if single line, bunch of segments otherwise.
  data_window_t::const_iterator it = data.begin();
  int x1, x2, y1, y2;
  x1 = it->x;
  y1 = it->y;
  it++;
  
  QPen pen = P.pen();
  pen.setStyle( Ps );
  pen.setWidth( 2 );
  pen.setColor( C );
//  P.setPen( Cz);
  P.setPen( pen );
  
  if(it == data.end()) {
    // 'line' has just a single point: plot blotch @ (x1, y1)
    sd_line_t::drawBigPoint(P, x1 , y1);
  } else {
    // draw segments between consecutive pixel pairs in list
    for(; it!=data.end(); it++) {
      x2 = it->x;
      y2 = it->y;
      P.drawLine(x1, y1, x2, y2);
      x1 = x2;
      y1 = y2;
    }
  }
} // --line_window_t::plot()


void line_window_t::show() {
  for(data_window_t::const_iterator it = data.begin(); it != data.end(); it++) {
    std::cerr << "(" << (it->x) << "," << (it->y) << ") "; 
  }
  std::cerr << " ### ";
} // --line_window_t::show()
